@@ -22,8 +22,6 @@ const DateMenu = imports.ui.dateMenu;
 const Main = imports.ui.main;
 const Tweener = imports.ui.tweener;
 
-const PANEL_HEIGHT = 26;
-
 const PANEL_ICON_SIZE = 24;
 
 const STARTUP_ANIMATION_TIME = 0.2;
@@ -33,8 +31,7 @@ const HOT_CORNER_ACTIVATION_TIMEOUT = 0.5;
 const BUTTON_DND_ACTIVATION_TIMEOUT = 250;
 
 const ANIMATED_ICON_UPDATE_TIMEOUT = 100;
-const SPINNER_UPDATE_TIMEOUT = 130;
-const SPINNER_SPEED = 0.02;
+const SPINNER_ANIMATION_TIME = 0.2;
 
 const STANDARD_TRAY_ICON_ORDER = ['a11y', 'display', 'keyboard', 'volume', 'bluetooth', 'network', 'battery'];
 const STANDARD_TRAY_ICON_SHELL_IMPLEMENTATION = {
@@ -47,6 +44,12 @@ const STANDARD_TRAY_ICON_SHELL_IMPLEMENTATION = {
 if (Config.HAVE_BLUETOOTH)
     STANDARD_TRAY_ICON_SHELL_IMPLEMENTATION['bluetooth'] = imports.ui.status.bluetooth.Indicator;
 
+try {
+    STANDARD_TRAY_ICON_SHELL_IMPLEMENTATION['network'] = imports.ui.status.network.NMApplet;
+} catch(e) {
+    log('NMApplet is not supported. It is possible that your NetworkManager version is too old');
+}
+
 // To make sure the panel corners blend nicely with the panel,
 // we draw background and borders the same way, e.g. drawing
 // them as filled shapes from the outside inwards instead of
@@ -244,6 +247,10 @@ AppMenuButton.prototype = {
 
         let bin = new St.Bin({ name: 'appMenu' });
         this.actor.set_child(bin);
+
+        this.actor.reactive = false;
+        this._targetIsCurrent = false;
+
         this._container = new Shell.GenericContainer();
         bin.set_child(this._container);
         this._container.connect('get-preferred-width', Lang.bind(this, this._getContentPreferredWidth));
@@ -271,20 +278,13 @@ AppMenuButton.prototype = {
             this.hide();
         }));
 
-        this._updateId = 0;
-        this._animationStep = 0;
-        this._clipWidth = PANEL_ICON_SIZE;
-        this._direction = SPINNER_SPEED;
+        this._stop = true;
 
         this._spinner = new AnimatedIcon('process-working.svg',
                                          PANEL_ICON_SIZE);
         this._container.add_actor(this._spinner.actor);
         this._spinner.actor.lower_bottom();
 
-        this._shadow = new St.Bin({ style_class: 'label-real-shadow' });
-        this._shadow.hide();
-        this._container.add_actor(this._shadow);
-
         let tracker = Shell.WindowTracker.get_default();
         tracker.connect('notify::focus-app', Lang.bind(this, this._sync));
         tracker.connect('app-state-changed', Lang.bind(this, this._onAppStateChanged));
@@ -300,6 +300,10 @@ AppMenuButton.prototype = {
 
         this._visible = true;
         this.actor.show();
+
+        if (!this._targetIsCurrent)
+            return;
+
         Tweener.removeTweens(this.actor);
         Tweener.addTween(this.actor,
                          { opacity: 255,
@@ -312,6 +316,11 @@ AppMenuButton.prototype = {
             return;
 
         this._visible = false;
+        if (!this._targetIsCurrent) {
+            this.actor.hide();
+            return;
+        }
+
         Tweener.removeTweens(this.actor);
         Tweener.addTween(this.actor,
                          { opacity: 0,
@@ -334,62 +343,26 @@ AppMenuButton.prototype = {
             this._iconBox.remove_clip();
     },
 
-    _stopAnimation: function(animate) {
-        this._label.actor.remove_clip();
-        if (this._updateId) {
-            this._shadow.hide();
-            if (animate) {
-                Tweener.addTween(this._spinner.actor,
-                                 { opacity: 0,
-                                   time: 0.2,
-                                   transition: "easeOutQuad",
-                                   onCompleteScope: this,
-                                   onComplete: function() {
-                                       this._spinner.actor.opacity = 255;
-                                       this._spinner.actor.hide();
-                                   }
-                                 });
-            }
-            Mainloop.source_remove(this._updateId);
-            this._updateId = 0;
-        }
-        if (!animate)
-            this._spinner.actor.hide();
-    },
-
     stopAnimation: function() {
-        this._direction = SPINNER_SPEED * 3;
-        this._stop = true;
-    },
+        if (this._stop)
+            return;
 
-    _update: function() {
-        this._animationStep += this._direction;
-        if (this._animationStep > 1 && this._stop) {
-            this._animationStep = 1;
-            this._stopAnimation(true);
-            return false;
-        }
-        if (this._animationStep > 1)
-            this._animationStep = 1;
-        this._clipWidth = this._label.actor.width - (this._label.actor.width - PANEL_ICON_SIZE) * (1 - this._animationStep);
-        if (this.actor.get_direction() == St.TextDirection.LTR) {
-            this._label.actor.set_clip(0, 0, this._clipWidth + this._shadow.width, this.actor.height);
-        } else {
-            this._label.actor.set_clip(this._label.actor.width - this._clipWidth, 0, this._clipWidth, this.actor.height);
-        }
-        this._container.queue_relayout();
-        return true;
+        this._stop = true;
+        Tweener.addTween(this._spinner.actor,
+                         { opacity: 0,
+                           time: SPINNER_ANIMATION_TIME,
+                           transition: "easeOutQuad",
+                           onCompleteScope: this,
+                           onComplete: function() {
+                               this._spinner.actor.opacity = 255;
+                               this._spinner.actor.hide();
+                           }
+                         });
     },
 
     startAnimation: function() {
-        this._direction = SPINNER_SPEED;
-        this._stopAnimation(false);
-        this._animationStep = 0;
-        this._update();
         this._stop = false;
-        this._updateId = Mainloop.timeout_add(SPINNER_UPDATE_TIMEOUT, Lang.bind(this, this._update));
         this._spinner.actor.show();
-        this._shadow.show();
     },
 
     _getContentPreferredWidth: function(actor, forHeight, alloc) {
@@ -451,18 +424,13 @@ AppMenuButton.prototype = {
         this._label.actor.allocate(childBox, flags);
 
         if (direction == St.TextDirection.LTR) {
-            childBox.x1 = Math.floor(iconWidth / 2) + this._clipWidth + this._shadow.width;
+            childBox.x1 = Math.floor(iconWidth / 2) + this._label.actor.width;
             childBox.x2 = childBox.x1 + this._spinner.actor.width;
             childBox.y1 = box.y1;
             childBox.y2 = box.y2 - 1;
             this._spinner.actor.allocate(childBox, flags);
-            childBox.x1 = Math.floor(iconWidth / 2) + this._clipWidth + 2;
-            childBox.x2 = childBox.x1 + this._shadow.width;
-            childBox.y1 = box.y1;
-            childBox.y2 = box.y2 - 1;
-            this._shadow.allocate(childBox, flags);
         } else {
-            childBox.x1 = this._label.actor.width - this._clipWidth - this._spinner.actor.width;
+            childBox.x1 = -this._spinner.actor.width;
             childBox.x2 = childBox.x1 + this._spinner.actor.width;
             childBox.y1 = box.y1;
             childBox.y2 = box.y2 - 1;
@@ -501,13 +469,6 @@ AppMenuButton.prototype = {
                 lastStartedApp = this._startingApps[i];
 
         let focusedApp = tracker.focus_app;
-        let targetApp = focusedApp != null ? focusedApp : lastStartedApp;
-        if (targetApp == this._targetApp) {
-            if (targetApp && targetApp.get_state() != Shell.AppState.STARTING)
-                this.stopAnimation();
-            return;
-        }
-        this._stopAnimation();
 
         if (!focusedApp) {
             // If the app has just lost focus to the panel, pretend
@@ -517,27 +478,56 @@ AppMenuButton.prototype = {
                 return;
         }
 
+        let targetApp = focusedApp != null ? focusedApp : lastStartedApp;
+
+        if (targetApp == null) {
+            if (!this._targetIsCurrent)
+                return;
+
+            this.actor.reactive = false;
+            this._targetIsCurrent = false;
+
+            Tweener.removeTweens(this.actor);
+            Tweener.addTween(this.actor, { opacity: 0,
+                                           time: Overview.ANIMATION_TIME,
+                                           transition: 'easeOutQuad' });
+            return;
+        }
+
+        if (!this._targetIsCurrent) {
+            this.actor.reactive = true;
+            this._targetIsCurrent = true;
+
+            Tweener.removeTweens(this.actor);
+            Tweener.addTween(this.actor, { opacity: 255,
+                                           time: Overview.ANIMATION_TIME,
+                                           transition: 'easeOutQuad' });
+        }
+
+        if (targetApp == this._targetApp) {
+            if (targetApp && targetApp.get_state() != Shell.AppState.STARTING)
+                this.stopAnimation();
+            return;
+        }
+
+        this._spinner.actor.hide();
         if (this._iconBox.child != null)
             this._iconBox.child.destroy();
         this._iconBox.hide();
         this._label.setText('');
-        this.actor.reactive = false;
 
         this._targetApp = targetApp;
-        if (targetApp != null) {
-            let icon = targetApp.get_faded_icon(2 * PANEL_ICON_SIZE);
+        let icon = targetApp.get_faded_icon(2 * PANEL_ICON_SIZE);
 
-            this._label.setText(targetApp.get_name());
-            // TODO - _quit() doesn't really work on apps in state STARTING yet
-            this._quitMenu.label.set_text(_("Quit %s").format(targetApp.get_name()));
+        this._label.setText(targetApp.get_name());
+        // TODO - _quit() doesn't really work on apps in state STARTING yet
+        this._quitMenu.label.set_text(_("Quit %s").format(targetApp.get_name()));
 
-            this.actor.reactive = true;
-            this._iconBox.set_child(icon);
-            this._iconBox.show();
+        this._iconBox.set_child(icon);
+        this._iconBox.show();
 
-            if (targetApp.get_state() == Shell.AppState.STARTING)
-                this.startAnimation();
-        }
+        if (targetApp.get_state() == Shell.AppState.STARTING)
+            this.startAnimation();
 
         this.emit('changed');
     }
@@ -641,12 +631,17 @@ PanelCorner.prototype = {
  * This class manages the "hot corner" that can toggle switching to
  * overview.
  */
-function HotCorner() {
-    this._init();
+function HotCorner(button) {
+    this._init(button);
 }
 
 HotCorner.prototype = {
-    _init : function() {
+    _init : function(button) {
+        // This is the activities button associated with this hot corner,
+        // if this is on the primary monitor (or null with the corner is
+        // on a different monitor)
+        this._button = button;
+
         // We use this flag to mark the case where the user has entered the
         // hot corner and has not left both the hot corner and a surrounding
         // guard area (the "environs"). This avoids triggering the hot corner
@@ -673,6 +668,8 @@ HotCorner.prototype = {
 
         this._activationTime = 0;
 
+        this.actor.connect('enter-event',
+                           Lang.bind(this, this._onEnvironsEntered));
         this.actor.connect('leave-event',
                            Lang.bind(this, this._onEnvironsLeft));
         // Clicking on the hot corner environs should result in the same bahavior
@@ -725,6 +722,8 @@ HotCorner.prototype = {
                                         x: x,
                                         y: y });
         ripple._opacity =  startOpacity;
+        if (ripple.get_direction() == St.TextDirection.RTL)
+            ripple.set_anchor_point_from_gravity(Clutter.Gravity.NORTH_EAST);
         Tweener.addTween(ripple, { _opacity: finalOpacity,
                                    scale_x: finalScale,
                                    scale_y: finalScale,
@@ -747,6 +746,11 @@ HotCorner.prototype = {
         this._addRipple(0.35,  1.0,   0.0,   0.3,    1,    0.0);
     },
 
+    _onEnvironsEntered : function() {
+        if (this._button)
+            this._button.hover = true;
+    },
+
     _onCornerEntered : function() {
         if (!this._entered) {
             this._entered = true;
@@ -774,6 +778,9 @@ HotCorner.prototype = {
     },
 
     _onEnvironsLeft : function(actor, event) {
+        if (this._button)
+            this._button.hover = false;
+
         if (event.get_related() != this._corner)
             this._entered = false;
         return false;
@@ -808,6 +815,8 @@ Panel.prototype = {
             this.actor.remove_style_class_name('in-overview');
         }));
 
+        this._leftPointerBarrier = 0;
+        this._rightPointerBarrier = 0;
         this._menus = new PopupMenu.PopupMenuManager(this);
 
         this._leftBox = new St.BoxLayout({ name: 'panelLeft' });
@@ -991,7 +1000,7 @@ Panel.prototype = {
                                                   affectsStruts: false,
                                                   affectsInputRegion: false });
 
-        Main.ctrlAltTabManager.addGroup(this.actor, _("Panel"), 'start-here',
+        Main.ctrlAltTabManager.addGroup(this.actor, _("Top Bar"), 'start-here',
                                         { sortGroup: CtrlAltTab.SortGroup.TOP });
     },
 
@@ -1068,7 +1077,21 @@ Panel.prototype = {
         let primary = global.get_primary_monitor();
 
         this.actor.set_position(primary.x, primary.y);
-        this.actor.set_size(primary.width, PANEL_HEIGHT);
+        this.actor.set_size(primary.width, -1);
+
+        if (this._leftPointerBarrier)
+            global.destroy_pointer_barrier(this._leftPointerBarrier);
+        if (this._rightPointerBarrier)
+            global.destroy_pointer_barrier(this._rightPointerBarrier);
+
+        this._leftPointerBarrier =
+            global.create_pointer_barrier(primary.x, primary.y,
+                                          primary.x, primary.y + this.actor.height,
+                                          1 /* BarrierPositiveX */);
+        this._rightPointerBarrier =
+            global.create_pointer_barrier(primary.x + primary.width, primary.y,
+                                          primary.x + primary.width, primary.y + this.actor.height,
+                                          4 /* BarrierNegativeX */);
 
         this._leftCorner.relayout();
         this._rightCorner.relayout();
